import logging
from abc import abstractmethod

from spaceone.core.manager import BaseManager

__all__ = ["PluginManager"]
_LOGGER = logging.getLogger(__name__)


class PluginManager(BaseManager):
    def register_plugin(self, params):
        pass

    def update_plugin(self, params):
        pass

    def enable_plugin(self, plugin_id, domain_id):
        pass

    def disable_plugin(self, plugin_id, domain_id):
        pass

    def delete_plugin(self, plugin_id, domain_id):
        pass

    @abstractmethod
    def get_plugin(self, repo_info: dict, plugin_id, domain_id):
        pass

    @abstractmethod
    def list_plugins(self, repo_info: dict, query: dict, params: dict):
        pass

    def stat_plugins(self, query):
        pass

    @abstractmethod
    def get_plugin_versions(self, repo_info: dict, plugin_id, domain_id):
        pass

    @staticmethod
    def change_response(
        info: dict, repo_info: dict = None, domain_id: str = None
    ) -> dict:
        if repo_info:
            info["repository_info"] = {
                "repository_id": repo_info["repository_id"],
                "name": repo_info["name"],
                "repository_type": repo_info["repository_type"],
            }

        if domain_id:
            info["domain_id"] = domain_id

        return info



import functools
from spaceone.api.repository.v1 import repository_pb2

__all__ = ["RepositoryInfo", "RepositoriesInfo"]


def RepositoryInfo(repository_info: dict, minimal=False):
    info = {
        "repository_id": repository_info["repository_id"],
        "name": repository_info["name"],
        "repository_type": repository_info["repository_type"],
    }

    if not minimal:
        info.update(
            {
                "endpoint": repository_info.get("endpoint"),
            }
        )

    return repository_pb2.RepositoryInfo(**info)


def RepositoriesInfo(repos_info, **kwargs):
    return repository_pb2.RepositoriesInfo(
        results=list(map(functools.partial(RepositoryInfo, **kwargs), repos_info)),
    )



DATABASES = {
    "default": {
        # 'db': 'repository',
        # 'host': 'localhost',
        # 'port': 27017,
        # 'username': 'repository',
        # 'password': ''
    }
}

CACHES = {"default": {}}

CONNECTORS = {
    "AWSPrivateECRConnector": {
        "aws_access_key_id": "",
        "aws_secret_access_key": "",
        "region_name": "",
        "account_id": "",
    },
    "DockerHubConnector": {},
    "HarborConnector": {
        "base_url": "",
        "token": "",
        "image_prefix": "",
        "verify": True,
    },
    "SpaceConnector": {
        "backend": "spaceone.core.connector.space_connector.SpaceConnector",
        "endpoints": {"identity": "grpc://identity:50051"},
    },
}

HANDLERS = {
    # "authentication": [{
    #     "backend": "spaceone.core.handler.authentication_handler.AuthenticationGRPCHandler",
    # }],
    # "authorization": [{
    #     "backend": "spaceone.core.handler.authorization_handler.AuthorizationGRPCHandler",
    #     "uri": "grpc://localhost:50051/v1/Authorization/verify"
    # }],
    # "mutation": [{
    #     "backend": "spaceone.core.handler.mutation_handler.SpaceONEMutationHandler"
    # }],
    # "event": []
}

REGISTRY_INFO = {
    "DOCKER_HUB": {"url": "registry.hub.docker.com"},
    "AWS_PRIVATE_ECR": {"url": "", "image_pull_secrets": ""},
    "HARBOR": {"url": "", "image_pull_secrets": ""},
}

# Use managed repository (Read Only), if you can not use plugin marketplace
ENABLE_MANAGED_REPOSITORY = False
DEFAULT_REGISTRY = "DOCKER_HUB"  # DOCKER_HUB | AWS_PRIVATE_ECR | HARBOR

ROOT_TOKEN = ""
ROOT_TOKEN_INFO = {}

MANAGED_REGISTRY_TYPE = "DOCKER_HUB"
MANAGED_REGISTRY_CONFIG = {}
MANAGED_PLUGIN_IMAGE_PREFIX = "spaceone"

# System Token
TOKEN = ""

REPOSITORIES = [
    {
        "repository_id": "repo-managed",
        "name": "Managed Repository",
        "repository_type": "MANAGED",
    },
    # {
    #     "repository_id": "repo-marketplace",
    #     "name": "marketplace",
    #     "repository_type": "REMOTE",
    #     "endpoint": "grpc://repository:50051",
    # },
    {
        "repository_id": "repo-local",
        "name": "Private",
        "repository_type": "LOCAL",
    },
]



import logging

from spaceone.core.service import *
from spaceone.repository.manager import (
    RepositoryManager,
)

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class RepositoryService(BaseService):
    resource = "Repository"

    @transaction(
        permission="repository:Repository.read",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )
    def list(self, params):
        """List repositories
        Args:
            params (dict):
                'repository_id': 'str',
                'repository_type': 'str'

        Returns:
            results (list): 'list of repository_vo'
            total_count (int)
        """
        repository_id: str = params.get("repository_id")
        repository_type: str = params.get("repository_type")
        repo_mgr: RepositoryManager = self.locator.get_manager("RepositoryManager")

        return repo_mgr.get_repositories(repository_id, repository_type)
